//	by EvilJess aka Jeffrey Bakker 2004

//	New or modified code (C)2004 Jeffrey Bakker <evil@damnclan.com>
//	Existing code for Unreal  Tournament 2003 Copyright  2002 Epic Games, Inc.
//	Existing code for Unreal  Tournament 2004 Copyright  2004 Epic Games, Inc.


//	BinStaGib - InstaGib meets BinSlayer
//	Copyright (C) 2004  Jeffrey Bakker

//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Lesser General Public
//	License as published by the Free Software Foundation; either
//	version 2.1 of the License, or (at your option) any later version.

//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Lesser General Public License for more details.

//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


class DeathSphere extends WeaponFire;

var float SMin;
var float SMax;
var float SGain;

var float mHoldClampMax;
var float mWaitTime;


auto state Idle
{
    function StopFiring()
    {
        GotoState('Wait');
        Super.StopFiring();
    }
}

state Wait
{
    function BeginState()
    {
        SetTimer(mWaitTime, false);
    }

    function Timer()
    {
        GotoState('LoadNext');
    }
}

state LoadNext
{
    function ModeTick(float dt)
    {
            GotoState('Idle');
    }
}

function PostBeginPlay()
{
	Super.PostBeginPlay();
	mHoldClampMax = (SMax - SMin) / SGain;
}

simulated function PlayFiring()
{
}

simulated function bool AllowFire()
{
	return true;
}

function ModeDoFire()
{
	Super.ModeDoFire();
}

function DoFireEffect()
{
	DeathSphereExplosion();
}

function DeathSphereExplosion()
{
	local Vector SphereLoc;
	local Rotator R;

	if (SMin + HoldTime * SGain < SMax) {
		PlaySound(Sound'WeaponSounds.RocketLauncher.SeekLost');
		return;
	}

	R = Rotation;
	R.Roll = Rand(360);

	SphereLoc = Instigator.Location;

	if ( (Instigator.PlayerReplicationInfo.Team != None) && (Instigator.PlayerReplicationInfo.Team.TeamIndex == 1) )
		Spawn(class'BinStaGib.SphereExplosionBlue',,, SphereLoc, R);
	else	Spawn(class'BinStaGib.SphereExplosion',,, SphereLoc, R);

	PlaySound(Sound'WeaponSounds.BExplosion5',, 255);
	DelayedHurtRadius(333, 444, Class'BinStaGib.DamTypeDeathSphere', 50000, SphereLoc );
}

simulated function DelayedHurtRadius( float DamageAmount, float DamageRadius, class<DamageType> DamageType, float Momentum, vector HitLocation )
{
	local actor Victims;
	local float damageScale, dist;
	local vector dir;
	
	if( bHurtEntry )
		return;

	bHurtEntry = true;
	foreach VisibleCollidingActors( class 'Actor', Victims, DamageRadius, HitLocation )
	{
		// don't let blast damage affect fluid - VisibleCollisingActors doesn't really work for them - jag
		if( (Victims != self) && (Victims.Role == ROLE_Authority) && (!Victims.IsA('FluidSurfaceInfo')) )
		{
			if(Pawn(Victims) != Instigator)
			{
				dir = Victims.Location - HitLocation;
				dist = FMax(1,VSize(dir));
				dir = dir/dist; 
				damageScale = 1 - FMax(0,(dist - Victims.CollisionRadius)/DamageRadius);
				if ( DamageType.Default.bDelayedDamage && (Instigator == None) && (Pawn(Victims) != None) )
					Pawn(Victims).DelayedDamageInstigatorController = Instigator.Controller;
				Victims.TakeDamage
				(
					damageScale * DamageAmount,
					Instigator, 
					Victims.Location - 0.5 * (Victims.CollisionHeight + Victims.CollisionRadius) * dir,
					(damageScale * Momentum * dir),
					DamageType
				);
			}
		} 
	}
	bHurtEntry = false;
}

defaultproperties
{
	SMin=200.000000
	SMax=666.000000
	SGain=237.000000
	mWaitTime=0.300000

	bFireOnRelease=True

	FireSound=Sound'WeaponSounds.BExplosion5'

	FireAnim=None
	FireRate=0.666666
	bModeExclusive=False
	BotRefireRate=0.060000
}